package gfcutil;

import GFC.Absyn.*;

import java.util.*;
import java.io.Serializable;

/**
 *  Represents a GFC function.
 */
public class Function implements Definition, Serializable {
        
        private String name;

        private List<String> argCats;

        private String resultCat;

        public Function(String name, Exp exp) {
                this.name = name;
                List<String> t = expToType(exp);
                this.resultCat = t.remove(t.size()-1);
                this.argCats = t;
        }

        /**
         *  Get the name of this function.
         */
        public String getName() {
                return name;
        }

        /**
         *  Get the argument types of this function.
         */
        public List<String> getArgCats() {
                return argCats;
        }


        public String toString() {
                StringBuilder sb = new StringBuilder();
                sb.append("fun ");
                sb.append(name);
                sb.append(" : ");
                for (String c : argCats) 
                        sb.append(c).append(" -> ");
                sb.append(resultCat);
                return sb.toString();
        }

        /**
         * FIXME: result should be a list of identifiers, not a list of strings
         */
        private static List<String> expToType(Exp exp) {
                return exp.accept(new ExpToType(), new LinkedList<String>());
        }


        private static class ExpToType implements Exp.Visitor<List<String>,List<String>> {
                private AtomToString atomToString = new AtomToString();

                public List<String>visit(GFC.Absyn.EApp p, List<String> arg) { return arg; }
                public List<String>visit(GFC.Absyn.EProd p, List<String> arg) {
                        p.exp_1.accept(this, arg);
                        return p.exp_2.accept(this, arg);
                }
                public List<String>visit(GFC.Absyn.EAbs p, List<String> arg) { return arg; }
                public List<String>visit(GFC.Absyn.EEq p, List<String> arg) { return arg; } 
                public List<String>visit(GFC.Absyn.EAtom p, List<String> arg) { 
                        String s = p.atom_.accept(atomToString, null);
                        arg.add(s);
                        return arg;
                }
                public List<String>visit(GFC.Absyn.EData p, List<String> arg) { return arg; }
        }

        private static class AtomToString implements Atom.Visitor<String,Object> {
                public String visit(GFC.Absyn.AC p, Object arg) { 
                        CIQ ciq = (CIQ)p.cident_;
                        return ciq.ident_1 + "." + ciq.ident_2;
                }
                public String visit(GFC.Absyn.AD p, Object arg) {
                        CIQ ciq = (CIQ)p.cident_;
                        return "<" + ciq.ident_1 + "." + ciq.ident_2 + ">";
                }
                public String visit(GFC.Absyn.AV p, Object arg) {
                        return "$" + p.ident_;
                }
                public String visit(GFC.Absyn.AM p, Object arg) {
                        return "?" + String.valueOf(p.integer_);
                }
                public String visit(GFC.Absyn.AS p, Object arg) {
                        return '"' + p.string_ + '"'; //FIXME: escape
                }
                public String visit(GFC.Absyn.AI p, Object arg) {
                        return String.valueOf(p.integer_);
                }
                public String visit(GFC.Absyn.AT p, Object arg) {
                        return "Type";
                }
        }

}
