package linearize;

import abssyn.*;
import gfcutil.*;
import linearize.gfvalue.*;
import GFC.*;
import GFC.Absyn.*;
import java.util.*;
import java.io.Serializable;

/**
 *  A linearizer which interprets a concrete GFC module.
 */
public class GFCLinearizer implements Serializable,Linearizer {

        private GFCModule module;

        /**
         *  @param module A GFC module
         */
        public GFCLinearizer(GFCModule module) {
                this.module = module;
        }

        public Value linearize(Tree t) {
                return t.accept(new TreeLinearizer(), this);
        }

        public String getName() {
                return module.getName();
        }

        public boolean tryThisOne() {
                return true;
        }

        /**
         *  Linearizes an abstract syntax tree by recursively applying 
         *  the linearization rules for each function.
         */
        private static class TreeLinearizer implements TreeVisitor<Value,GFCLinearizer> {

                public Value visit(Fun t, GFCLinearizer rules) {
                        Tree[] c = t.getChildren();
                        Value[] args = new Value[c.length];
                        for (int i = 0; i < c.length; i++)
                                args[i] = c[i].accept(this, rules);
                        String label = t.getLabel();
                        LinRule rule = rules.module.findLinRule(label);

                        if (rule.getArity() != args.length)
                                throw new RuntimeException(label + " requires " + rule.getArity() 
                                                           + " arguments, got " + args.length);
                        return rule.getTerm().accept(new InterpretLinRule(), args);
                }

                public Value visit(Literal l, GFCLinearizer rules) {
                        return Record.ss(l.getValue());
                }

                public Value visit(MetaVariable v, GFCLinearizer rules) {
                        // FIXME: use lindef
                        return Record.ss("? : " + v.getType());
                }

        }

}
