package linearize;

import java.util.*;
import java.io.Serializable;

/**
 *  A container for a number of linearizers.
 */
public class Linearizers implements Serializable {

        /**
         *  Maps concrete grammar names (language names) to linearizers.
         */
        private Map<String,Linearizer> map;

        public Linearizers() {
                this.map = new TreeMap<String,Linearizer>();
        }

        /**
         *  Get a linearizer by (language) name.
         */
        public Linearizer getLinearizer(String lang) {
                Linearizer l = map.get(lang);

                if (l == null) 
                        throw new RuntimeException("There is no linearizer for " + lang);

                return l;
        }

        /**
         *  List the (language) names of all the linearizers.
         */
        public Set<String> listLinearizers() {
                return map.keySet();
        }

        public void addLinearizer(Linearizer l) {
                String lang = l.getName();

                if (map.containsKey(lang))
                        throw new RuntimeException("There is already a linearizer for " + lang);

                map.put(lang, l);
        }

}
