package parse;

import abssyn.Tree;
import util.Pair;
import static util.Pair.pair;

import java.util.*;

public class ParseAll implements Parser {

        private Parsers parsers;

        public ParseAll(Parsers parsers) {
                this.parsers = parsers;
        }

        public List<Tree> parse(String str) {
                List<Pair<String,Tree>> ts = parseWithAll(str);
                List<Tree> rs = new LinkedList<Tree>();
                for (Pair<String,Tree> p : ts) 
                        rs.add(p.snd);
                return rs;
        }

        /**
         *  Ignores the category argument.
         */
        public List<Tree> parse(String str, String cat) {
                return parse(str);
        }

        public String getName() {
                return "*all*";
        }

        public boolean tryThisOne() {
                return false;
        }

        /**
         *  Try parsing the input with all the available parsers.
         *
         *  @return A list of pairs (language, parse tree)
         */
        public List<Pair<String,Tree>> parseWithAll(String str) {
                System.err.println("Trying to parse '" + str + "'");

                List<Pair<String,Tree>> rs 
                        = new LinkedList<Pair<String,Tree>>();
                for (String l: parsers.listParsers()) {
                        Parser p = parsers.getParser(l);
                        if (p.tryThisOne()) {
                                System.err.println("Trying " + l);
                                List<Tree> ts = p.parse(str);
                                for (Tree t : ts) {
                                        System.err.println(l + ": " + t);
                                        rs.add(pair(l, t));
                                }
                        }
                }
                return rs;
        }

}
