package parse;

import abssyn.*;

import java.util.List;

/**
 *  Interface for all parsers from text to abstract syntax.
 */
public interface Parser {

        /**
         *  Parse a string to zero or more abstract syntax trees the
         *  parsers default category.
         */
        public List<Tree> parse(String str);

        /**
         *  Parse a string to zero or more abstract syntax trees in
         *  the given category.
         */
        public List<Tree> parse(String str, String cat);

        /**
         *  Get the name of this parser.
         */
        public String getName();

        /**
         *  Whether this parser should be used when trying to parse a 
         *  string with different parsers. This can be used to exclude
         *  special parsers from such attempts.
         */ 
        public boolean tryThisOne();

}
