package parse.chart;

import CFG.*;
import CFG.Absyn.*;
import util.*;

import java.util.*;
import java.io.*;

public class CFGGrammars {

        private Map<String,CFGrammar> grammars;

        public CFGGrammars(Grammars gs) {
                grammars = new HashMap<String,CFGrammar>();

                for (Grammar g : gs.listgrammar_) {
                        CFGrammar cf = makeCFGrammar(g);
//                        System.err.println(cf);
                        grammars.put(cf.getName(), cf);
                }
        }

        public Set<String> listGrammars() {
                return grammars.keySet();
        }

        public CFGrammar getGrammar(String name) {
                return grammars.get(name);
        }

        private CFGrammar makeCFGrammar(Grammar g) {
                String start = getStartCat(g);
                CFGrammar cf = new CFGrammar(g.ident_, new parse.chart.Category(start));

                for (CFG.Absyn.Rule r : g.listrule_) {
                        String fun = r.ident_;
                        String name = fun + ":" + nameToString(r.name_);
                        int[][] profile = getProfile(r.profile_);
                        Category cat = new Category(categoryToString(r.category_));
                        List<parse.chart.Symbol> rhs = new LinkedList<parse.chart.Symbol>();
                        for (CFG.Absyn.Symbol s : r.listsymbol_)
                                rhs.add(symbolToSymbol(s));
                        cf.addRule(name, cat, rhs, fun, profile);
                }

                return cf;
        }

        private String getStartCat(Grammar g) {
                String cat = "S{}.s";
                GetStartCat visitor = new GetStartCat();
                for (Flag f : g.listflag_)
                        cat = f.accept(visitor, cat);
                return cat;
        }

        private static class GetStartCat implements Flag.Visitor<String,String> {
                public String visit(CFG.Absyn.StartCat p, String arg) {
                        return categoryToString(p.category_);
                }
        }


        public static CFGGrammars readCFGM(Reader in) throws IOException {
                System.err.println("Parsing CFGM file");
                Grammars gs = CFGParser.parse(in);
                in.close();
                System.err.println("Done parsing CFGM file");
                return new CFGGrammars(gs);
        }

        public static CFGGrammars readCFGM(String file) throws IOException {
                return readCFGM(FileUtil.openFile(file));
        }

        private int[][] getProfile(Profile in) {
                int[][] profile = new int[in.listints_.size()][];
                int i = 0;
                for (Ints is : in.listints_) {
                        profile[i] = new int[is.listinteger_.size()];
                        int j = 0;
                        for (int n : is.listinteger_) {
                                profile[i][j] = n;
                                j++;
                        }
                        i++;
                }
                return profile;
        }

        private parse.chart.Symbol symbolToSymbol(CFG.Absyn.Symbol in) {
                return in.accept(new SymbolToSymbol(), null);
        }

        private class SymbolToSymbol implements CFG.Absyn.Symbol.Visitor<parse.chart.Symbol,Object> {
                public parse.chart.Symbol visit(CFG.Absyn.CatS p, Object arg) {
                        return new parse.chart.Category(categoryToString(p.category_));
                }
                public parse.chart.Symbol visit(CFG.Absyn.TermS p, Object arg) {
                        return new Terminal(p.string_);
                }
        }

        private static String categoryToString(CFG.Absyn.Category cat) {
                return PrettyPrinter.print(cat);
        }

        private static String nameToString(CFG.Absyn.Name name) {
                return PrettyPrinter.print(name);
        }

}
