package parse.chart;

import java.util.*;

public class CFGrammar {

        private String name;

        private Category start;

        private List<NonTerminalRule> nonTerminalRules;

        // FIXME: should maybe be a trie
        private Set<TerminalRule> terminalRules;

        private Set<EmptyRule> emptyRules;

        public CFGrammar(String name, Category start) {
                this.name = name;
                this.start = start;
                this.nonTerminalRules = new LinkedList<NonTerminalRule>();
                this.terminalRules = new HashSet<TerminalRule>();
                this.emptyRules = new HashSet<EmptyRule>();
        }

        public String getName() {
                return name;
        }

        public Category getStart() {
                return start;
        }

        /**
         *  Get the rules with only non-terminals in the RHS.
         */
        public Collection<NonTerminalRule> getNonTerminalRules() {
                return nonTerminalRules;
        }

        /**
         *  Get the rules with only a single terminal in the RHS.
         */
        public Collection<TerminalRule> getTerminalRules() {
                return terminalRules;
        }

        /**
         *  Get the empty rules.
         */
        public Collection<EmptyRule> getEmptyRules() {
                return emptyRules;
        }

        public void addRule(String name, Category cat, List<Symbol> rhs, String fun, int[][] profile) {
                if (rhs.isEmpty()) {
                        EmptyRule r = new EmptyRule(name, cat, fun, profile.length);
//                        System.err.println("Added empty rule: " + r);
                        emptyRules.add(r);
                } else {
                        // FIXME: special case for rules that are already single-terminal rule?

                        List<Category> rhsc = new LinkedList<Category>();

                        for (Symbol s : rhs) {
                                if (s instanceof Terminal) {
                                        Terminal t = (Terminal)s;
                                        Category c = addTerminal(t);
                                        rhsc.add(c);
                                } else {
                                        rhsc.add((Category)s);
                                }
                        }
                        
                        NonTerminalRule r = new NonTerminalRule(name, cat, rhsc, fun, profile);
                        nonTerminalRules.add(r);
//                System.err.println("Added non-terminal rule: " + r);
                }
        }

        /**
         *  Add a terminal rule.
         */
        private Category addTerminal(Terminal t) {
                Category cat = makeTerminalCategory(t);
                TerminalRule r = new TerminalRule(cat.getName(), cat, t);
                terminalRules.add(r);
//                System.err.println("Added terminal rule: " + r);
                return cat;
        }

        private Category makeTerminalCategory(Terminal t) {
                return new Category("$" + t.toString()); //FIXME: might have to be more advanced
        }

        public String toString() {
                StringBuilder sb = new StringBuilder();
                sb.append("grammar ").append(name).append("\n");
                sb.append("startcat ").append(start).append("\n");
                for (NonTerminalRule r : nonTerminalRules) 
                        sb.append(r.toString()).append("\n");
                for (TerminalRule r : terminalRules) 
                        sb.append(r.toString()).append("\n");
                sb.append("end grammar\n");
                return sb.toString();
        }
}
