package parse.chart;

import java.util.*;

public class Chart {

        private Set<Edge>[][] chart;

        public Chart(int n) {
                // causes an unchecked cast warning. The cast is neccessary
                // since we cannot create arrays of parametrized types.
                // See section 7.3 of
                // http://java.sun.com/j2se/1.5/pdf/generics-tutorial.pdf
                chart = (Set<Edge>[][])new Set<?>[n+1][n+1];
                for (int i = 0; i <= n; i++)
                        for (int j = i; j <= n; j++)
                                chart[i][j] = new HashSet<Edge>();
        }

        /**
         *  @return true if the edge was not already in the set, i.e. if it was added.
         */
        public boolean add(Edge e, int i, int j) {
                if (i > j)
                        throw new IllegalArgumentException("i > j");

                boolean addedNew = chart[i][j].add(e);
//                if (addedNew)
//                        System.err.println("("+i+","+j+"): Added: " + e);
                return addedNew;
        }

        public Set<Edge> get(int i, int j) {
                if (i > j)
                        throw new IllegalArgumentException("i > j");

                return chart[i][j];
        }

        /**
         *  Get the input length that this chart is for.
         */
        public int getSize() {
                return chart.length-1;
        }

        /**
         *  Remove all active edges.
         */
        public void purgeActive() {
                int n = chart.length-1;
                for (int i = 0; i <= n; i++) {
                        for (int j = i; j <= n; j++) {
                                Iterator<Edge> it = chart[i][j].iterator();
                                while (it.hasNext())
                                        if (it.next().isActive())
                                                it.remove();
                        }
                }
        }

}
