package parse.chart;

import abssyn.Tree;
import parse.Parser;

import java.util.List;


/**
 *  Combines lexing, kilbury parsing and tree building.
 */
public class ChartParser implements Parser {

        private CFGrammar grammar;

        private KilburyParser parser;

        public ChartParser(CFGrammar grammar) {
                this.grammar = grammar;
                parser = new KilburyParser(grammar);
        }

        public List<Tree> parse(String str) {
                return parse(str, grammar.getStart());
        }

        public List<Tree> parse(String str, String cat) {
                return parse(str, new Category(cat));
        }

        public List<Tree> parse(String str, Category cat) {
                Lexer l = new Lexer();
                List<Token> tokens = l.lex(str);
                Chart chart = parser.parse(tokens);


//                 System.err.println("Passive edges: ");
//                 int n = chart.getSize();
//                 for (int i = 0; i <= n; i++)
//                         for (int j = i; j <= n; j++)
//                                 for (Edge e : chart.get(i,j))
//                                         System.err.println("("+i+","+j+"): " + e);

//                System.out.println("Complete parses:");
//                for (Edge e : chart.get(0, chart.getSize()))
//                        System.out.println(e);

                List<Tree> trees = new TreeBuilder(chart).buildTree(cat);

                return trees;
        }

        public String getName() {
                return grammar.getName();
        }

        public boolean tryThisOne() {
                return true;
        }

}
