package parse.chart;

import java.util.*;

public class Edge {

        private Rule rule;

        /** Position of the dot in the rule. */
        private int pos;

        public Edge(Rule rule) {
                this.rule = rule;
                this.pos = 0;
        }

        public Edge(Edge e) {
                this.rule = e.rule;
                this.pos = e.pos;
        }

        public Category getCategory() {
                return rule.getCategory();
        }

        /**
         *  Get the rule that this edge is for.
         */
        public Rule getRule() {
                return rule;
        }

        /**
         *  Checks if the next symbol needed is the given category.
         */
        public boolean needs(Category c) {
                if (!isActive() || rule.isTerminal())
                        return false;

                return ((NonTerminalRule)rule).getRhs().get(pos).equals(c);
        }

        /**
         *  Checks if this edge is active, that is, if it is still missing 
         *  some symbol.
         */
        public boolean isActive() {
                return pos < rule.getSize();
        }

        /**
         *  Move the dot one step forward.
         */
        public void stepForward() {
                if (!isActive())
                        throw new IllegalStateException(
                                "Can't go forward in a passive rule.");
                pos++;
        }
        
        public boolean equals(Object o) {
                return o instanceof Edge && equals((Edge)o);
        }

        public boolean equals(Edge e) {
                return rule.equals(e.rule) && pos == e.pos;
        }

        public int hashCode() {
                return rule.hashCode() + pos;
        }

        public String toString() {
                StringBuilder sb = new StringBuilder();
                sb.append(rule);
                sb.append(" (").append(pos).append(")");
                return sb.toString();
        }

}
