package translate;

import abssyn.*;
import parse.*;
import linearize.*;
import linearize.gfvalue.*;

import java.util.*;

public class Translator {

        private Parsers parsers;

        private TypeAnnotator annotator;

        private Linearizers linearizers;


        public Translator(Parsers parsers, TypeAnnotator annotator, Linearizers linearizers) {
                this.parsers = parsers;
                this.annotator = annotator;
                this.linearizers = linearizers;
        }

        public Set<String> translate(String fromLang, String toLang, String text) {
                Parser p = parsers.getParser(fromLang);
                Linearizer l = linearizers.getLinearizer(toLang);
                return translate(p, l, text);
        }

        private Set<String> translate(Parser p, Linearizer l, String text) {
                List<Tree> trees = p.parse(text);
                Set<String> output = new HashSet<String>();

//                System.err.println("Annotated parse trees: ");

                for (Tree tree : trees) {
                        annotator.annotate(tree);
//                        System.err.println(tree);
                        Value v = l.linearize(tree);
                        output.add(valueAsString(v));
                }

                return output;
        }

        private String valueAsString(Value value) {
                 Record r = (Record)value;
                 Value v = r.get("s"); // FIXME: is there always an s field?
                 return v.toString(); // FIXME: pick the first form if it is a table
        }

        public Set<String> listParsers() {
                return parsers.listParsers();
        }

        public Set<String> listLinearizers() {
                return linearizers.listLinearizers();
        }

}
