package translategui;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.util.Set;

import translate.Translator;

public class TranslatePanel extends JPanel {

        private Translator translator;

        private JTextArea input;
        private JTextArea output;

        private JComboBox inputLang;
        private JComboBox outputLang;

        private JCheckBox preview;

        public TranslatePanel(Translator translator) {
                this.translator = translator;
                buildGUI();
        }

        private void buildGUI() {
                setBorder(BorderFactory.createMatteBorder(1, 1, 1, 1, Color.black));

                input = new JTextArea(4, 40);
                input.getDocument().addDocumentListener(new InputChangeListener());

                output = new JTextArea(10, 40);
                output.setEditable(false);

                JScrollPane outputScroll = new JScrollPane(output, 
                                                           JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
                                                           JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
                

                inputLang = new JComboBox();
                for (String l : translator.listParsers())
                        inputLang.addItem(l);
                inputLang.addActionListener(new TranslateListener());

                outputLang = new JComboBox();
                for (String l : translator.listLinearizers())
                        outputLang.addItem(l);
                outputLang.addActionListener(new TranslateListener());

                JButton transButt = new JButton("Translate!");
                transButt.addActionListener(new TranslateListener());

                preview = new JCheckBox("preview", false);
                preview.addActionListener(new TranslateListener());

                JPanel controlPane = new JPanel();
                controlPane.setBorder(BorderFactory.createMatteBorder(1, 0, 1, 0, Color.black));
                controlPane.add(new JLabel("Translate from "));
                controlPane.add(inputLang);
                controlPane.add(new JLabel(" to "));
                controlPane.add(outputLang);
                controlPane.add(transButt);
                controlPane.add(preview);

                JPanel inputPane = new JPanel(new BorderLayout());
                inputPane.add(input, BorderLayout.CENTER);
                inputPane.add(controlPane, BorderLayout.SOUTH);

                setLayout(new BorderLayout());
                add(inputPane, BorderLayout.NORTH);
                add(outputScroll, BorderLayout.CENTER);
        }


        private void doTranslate() {
                String fromLang = (String)inputLang.getSelectedItem();
                String toLang = (String)outputLang.getSelectedItem();
                String inputText = input.getText();
                String outputText;
                Set<String> outputs = translator.translate(fromLang, toLang, inputText);

                if (outputs.size() == 0) { // FIXME: handle these better
                        output.setText("<no parse>");
                } else if (outputs.size() > 1) {
                        output.setText("<ambiguous translation>");
                        System.err.println("Possible translations: ");
                        for (String s : outputs)
                                System.err.println(s);
                } else
                        output.setText(outputs.iterator().next());
        }

        private void doPreview() {
                if (preview.isSelected())
                        doTranslate();
        }

        private class TranslateListener implements ActionListener {
                public void actionPerformed(ActionEvent e) {
                        if (e.getSource() != preview || preview.isSelected())
                                doTranslate();
                }
        }

        private class InputChangeListener implements DocumentListener {
                public void changedUpdate(DocumentEvent e) { }
                public void insertUpdate(DocumentEvent e) { 
                        doPreview();
                }
                public void removeUpdate(DocumentEvent e) { 
                        doPreview();
                }
        }


}
